package com.xkrs.dao;

import com.xkrs.model.entity.FirePointEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author XinYi Song
 */
@Component
public interface FirePointDao extends JpaRepository<FirePointEntity,Long>, JpaSpecificationExecutor<FirePointEntity> {

    /**
     * 查询今天的火点信息
     * @param addTime
     * @return
     */
    @Query(value = "select * from fire_point where add_time like CONCAT('%',:addTime,'%')",nativeQuery = true)
    List<FirePointEntity> selectTodayFirePoint(String addTime);

    /**
     * 根据火点编码查询火点信息
     * @param fireCode
     * @return
     */
    FirePointEntity findByFireCode(String fireCode);

    /**
     * 根据火点编码修改火点状态
     * @param fireCode
     * @param fireType
     */
    @Modifying(clearAutomatically=true)
    @Query(value = "update fire_point set fire_type = ?2 where fire_code = ?1",nativeQuery = true)
    void updateFireTypeByFireCode(String fireCode,String fireType);
}
